// niz7.h - Genericka klasa nizova pokazivaca na podatke.

#ifndef _niz7_h_
#define _niz7_h_

#include <iostream>
using namespace std;

class GPun {};                     // KLASA GREAKA: Niz je pun.
inline ostream& operator<<(ostream& it, const GPun&)
  { return it << "*** Niz je pun! ***";}

class GIndeks {};                  // KLASA GREAKA: Nedozvoljen indeks.
inline ostream& operator<<(ostream& it, const GIndeks&)
  { return it << "*** Nedozvoljen indeks! ***"; }

class GPrazno {};                  // KLASA GREAKA: Mesto je prazno.
inline ostream& operator<<(ostream& it, const GPrazno&)
  { return it << "*** Mesto je prazno! ***"; }

template <class T>
class Niz {                             // GENERICKA KLASA NIZOVA:
  T** niz;                              // Niz pokazivaca na elemente.
  int kap;                              // Kapacitet niza.
  void kopiraj(const Niz& n);           // Kopiranje u objekat.
  void brisi();                         // Brisanje objekta.
public:
  explicit Niz(int k=10);               // Stvaranje praznog niza.
  Niz(const Niz& n) { kopiraj(n); }     // Konstruktor kopije.
  ~Niz() { brisi(); }                   // Destruktor.
  Niz& operator=(const Niz& n) {        // Dodela vrednosti.     
    if (this != &n) { brisi(); kopiraj(n); }
    return *this;
  }
  int kapac() const { return kap; }     // Kapacitet niza.
  Niz& operator+=(const T& t);          // Dodavanje elementa.
  Niz& izbaci(int i) {                  // Izbacivanje elementa.
    if (i>=0 && i<kap) { delete niz[i]; niz[i] = 0; }
    return *this;
  }
  T& operator[](int i) {                // Pristup elementu
    if (i<0 || i>=kap) throw GIndeks(); // - promenljivog niza,
    if (!niz[i]) throw GPrazno ();
    return *niz[i];
  }
  const T& operator[](int i) const {    // - nepromenljivog niza.
    if (i<0 || i>=kap) throw GIndeks();
    if (!niz[i]) throw GPrazno();
    return *niz[i];
  }
  template <class T>                    // Pisanje.
  friend ostream& operator<<(ostream& it, const Niz<T>& n);
};

template <class T>                      // METODE I FUNKCIJE UZ KLASU NIZOVA
void Niz<T>::kopiraj(const Niz& n) {    // Kopiranje u objekat.
  niz = new T* [kap = n.kap];
  for (int i=0; i<kap; i++) niz[i] = n.niz[i] ? n.niz[i]->kopija() : 0;
}

template <class T>
void Niz<T>::brisi() {                  // Brisanje objekta.
  for (int i=0; i<kap; delete niz[i++]);
  delete [] niz;
}

template <class T>                      // Stvaranje praznog niza.
Niz<T>::Niz(int k) {
  niz = new T* [kap = k];
  for (int i=0; i<k; niz[i++]=0);
}

template <class T>
Niz<T>& Niz<T>::operator+=(const T& t) {// Dodavanje elementa.
  int i = 0; while (i<kap && niz[i]) i++;
  if (i == kap) throw GPun();
  niz[i] = t.kopija();
  return *this;
}

template <class T>                      // Pisanje.
ostream& operator<<(ostream& it, const Niz<T>& n) {
  it << '<';
  for (int i=0; i<n.kap; i++) {
    if (i) it << " # ";
    if (n.niz[i]) it << *n.niz[i];
  }
  return it << '>';
}

#endif
